import docker
import os

from azure.core.exceptions import ResourceNotFoundError
from azure.storage.blob import BlobClient
from docker.errors import DockerException, NotFound

import time

import conduit_config

from log_config import logger


class ArtifactFetcher:

    def __init__(self, azure_storage_key, docker_password,
                 azure_account_name=conduit_config.get("bp_storage_account"),
                 bp_docker_user=conduit_config.get("system.docker.user"),
                 docker_registry=conduit_config.get("system.docker.registry")):
        self._azure_storage_key = azure_storage_key
        self._azure_account_name = azure_account_name
        self._docker_client = docker.from_env()
        logger.info("Logging into registry '%s' ", docker_registry)
        response = self._docker_client.login(registry=docker_registry,
                                             username=bp_docker_user, password=docker_password)
        if "errors" in response:
            logger.error("Errors during logging into repository '%s', errors: '%s' ", docker_registry, response['errors'])
        else:
            if "Status" in response:
                logger.info("Login response status: %s ", response['Status'])
            else:
                logger.info("Login without errors")

    def _build_azure_connection_string(self):
        return "DefaultEndpointsProtocol=https;AccountName={};AccountKey={};EndpointSuffix=core.windows.net".format(
            self._azure_account_name, self._azure_storage_key)

    def download_deployment_artifacts(self, target_version, dest_path):
        conduit_tar_gz = "builds/%s/conduit-%s.tar.gz" % (target_version, target_version)
        container_name = conduit_config.get("service.azure_storage.container_name")
        logger.info("Downloading %s into %s", conduit_tar_gz, dest_path)
        connection_str = self._build_azure_connection_string()

        try:
            blob_client = BlobClient.from_connection_string(conn_str=connection_str,
                                                            container_name=container_name, blob_name=conduit_tar_gz)
            # Download Append Blob
            with open(dest_path, "wb") as dest_file:
                download_stream = blob_client.download_blob()
                dest_file.write(download_stream.readall())
        finally:
            blob_client.close()

        if not os.path.isfile(dest_path):
            raise FileNotFoundError("Couldn't download artifacts from azure storage , dest_path=" + dest_path)

    def pull_docker_images(self, target_version):

        for retry in self._onerror_retry(DockerException, self._do_pull_docker_images):
            retry(target_version)

    def _do_pull_docker_images(self, target_version):
        try:
            logger.info("Pulling docker images, version: %s ", target_version)
            bp_docker_registry = conduit_config.get("system.docker.registry")
            self._docker_client.images.pull(repository="%s/bpcs/conduit-proxy" % bp_docker_registry, tag=target_version)
            self._docker_client.images.pull(repository="%s/bpcs/bde-server" % bp_docker_registry, tag=target_version)

        except DockerException as de:
            raise de

    def _onerror_retry(self, exception, callback, max_attempt=3, delay_secs=5):
        for retry in range(max_attempt):
            try:
                yield callback
                break
            except exception as e:
                if retry < max_attempt - 1:
                    time.sleep(delay_secs)
                else:
                    logger.exception("Failed to pull application images", exc_info=e)
                    raise e

    def docker_image_exists(self, repo, tag):
        bp_docker_registry = conduit_config.get("system.docker.registry")
        try:
            image_metadata = self._docker_client.images.get_registry_data(name="%s/%s:%s" % (bp_docker_registry, repo, tag))
            if image_metadata:
                return True
            else:
                return False

        except NotFound as nf:
            return False

    def azure_artifact_exists(self, target_version):
        conduit_tar_gz = "builds/%s/conduit-%s.tar.gz" % (target_version, target_version)
        container_name = conduit_config.get("service.azure_storage.container_name")
        connection_str = self._build_azure_connection_string()
        try:
            with BlobClient.from_connection_string(conn_str=connection_str,
                                                   container_name=container_name,
                                                   blob_name=conduit_tar_gz) as blob_client :
                blob_client.get_blob_properties()
                return True
        except ResourceNotFoundError as ne:
            pass

        return False
