import os
from urllib.parse import urlparse
import socket

settings = {
    "bp_storage_account": "bpartifactstorage",
    "system.settings.location": "/etc/bpcs",
    "system.backup.location": '{{ conduit_backup_system_storage | default("/opt/bpcs/backup") }}',
    "container.backup.location": '/usr/share/elasticsearch/backup',
    "service.azure_storage.container_name": "artifacts",
    "service.workdir": "{{ upgrade_service_workdir | default('~/.conduit') }}",
    "service.server.ip": "{{ upgrade_server_bind_host | default('localhost') }}",
    "service.server.port": "{{ upgrade_server_port | default('3005') }}",
    "service.server.vms": "{{ deployment_vms_hosts | default( groups['bde_host'] | join(',') ) }}",
    "system.indexes": ["data_source_metadata", "bdemeta_namespace", "bde_user", "bde_license",
                       "persistent_config", "cluster_config", "ad_subscriptions", "bde_group"],
    "system.indexes.backup.exclude": ["*", "-query_audit", "-logstash*", "-.apm*", "-.async-search", "-kibana*", "-.kibana*"
        , "-ilm-history*", "-.slm-history-*"],
    "system.indexes.backup_repo": "conduit_system_update_backup",
    "system.docker.registry": "{{ docker_registry | default('blueprint.azurecr.io') }}",
    "system.docker.user": "{{ docker_user | default('conduit-onprem') }}",
    "system.deployment_api.url": "{{ deployment_api_url | default('http://localhost:8080/') }}",
    "system.deployment_api.auth_token": "{{ conduit_deployment_auth_token }}",
    "system.bde-server.private_url": "{{ private_conduit_url | default('http://localhost:3000') }}",
    "system.bde-server.public_url": "{{ conduit_url | default('https://localhost/') }}",
    "system.bde-server.license_token": "{{ license_token | default('changeit') }}",
    "service.docker_compose_dir": "{{ docker_compose_dir | default('/etc/bpcs/docker') }}",
    "service.docker_compose_files": "{{ docker_compose_files | default('docker-compose.yml,docker-compose-worker.yml') }}",
    "service.elasticsearch.keystore.importer": '/usr/share/elasticsearch/conduit-keystore-importer.sh',
    "service.snapshot.repository.name": 'conduit_snapshot_repository',
    "service.elasticsearch.host": '{{ elasticsearch_host | default("localhost:9200") }}'
}


def get(prop_name: str):
    return settings[prop_name]


def get_elasticsearch_host():
    return get("service.elasticsearch.host")


def get_system_backup_location():
    return get("system.backup.location")


def get_deployment_vms():
    vms = get("service.server.vms").split(",")
    # if vms contains only localhost means we are on singlebox deployment, we'll change localhost with hostname
    if len(vms) == 1 and vms[0] == "localhost":
        return [socket.gethostname()]
    else:
        return vms


def get_container_backup_location():
    return get("container.backup.location")


def get_system_snapshots_location():
    return os.path.join(get("system.backup.location"), "snapshots")


def get_container_snapshots_location():
    return os.path.join(get("container.backup.location"), "snapshots")


def get_es_keystore_importer_path():
    return get("service.elasticsearch.keystore.importer")


def get_elasticsearch_repository():
    return get("service.snapshot.repository.name")


def get_system_setting_location(backup_name: str):
    return os.path.join(get("service.workdir"), backup_name, "settings")


def get_ansible_script_location(upgrade_run_id: str, playbook_name: str = None):
    if playbook_name:
        return os.sep.join((work_dir(), upgrade_run_id, "ansible", playbook_name))
    else:
        return os.sep.join((work_dir(), upgrade_run_id, "ansible"))


def get_ansible_inventory(upgrade_run_id: str, inventory_name: str = "local_hosts"):
    return os.sep.join((work_dir(), upgrade_run_id, "ansible", "inventory", inventory_name))


def work_dir():
    return os.path.expanduser(get("service.workdir"))


def get_ansible_archive_dest(upgrade_run_id: str):
    return os.sep.join((work_dir(), upgrade_run_id, "conduit.tar.gz"))


def get_system_indexes():
    return get("system.indexes")


def get_system_backup_indexes():
    return get("system.indexes.backup.exclude")


def get_system_indexes_backup_repo():
    return get("system.indexes.backup_repo")


def get_conduit_license_token():
    return get("system.bde-server.license_token")


def get_deployment_token():
    return get("system.deployment_api.auth_token")


def get_conduit_fqdn():
    public_url = get("system.bde-server.public_url")
    parsed_url = urlparse(public_url)
    return parsed_url.hostname


def get_bde_server_internal_url():
    return get("system.bde-server.private_url")


def get_docker_compose_dir():
    return get("service.docker_compose_dir")


def get_docker_compose_files():
    docker_compose_files = get("service.docker_compose_files")
    if docker_compose_files:
        return docker_compose_files.split(",")
    else:
        return None
