from urllib.parse import urljoin

import requests
from requests import HTTPError

from common import DeploymentStatus, UpgradeContext
from http_utils import HttpUtils

from log_config import logger


class ApiSession(object):
    def __init__(self, auth_token=None, http_session=None, ssl_verify=False):
        self._auth_token = auth_token
        self.sslVerify = ssl_verify
        if http_session is None:
            self.httpSession = requests.Session()
        else:
            self.httpSession = http_session

        self.httpSession.verify = self.sslVerify

    def add_auth_token(self, headers):
        if self._auth_token:
            headers['Authorization'] = self._auth_token


class DeploymentApiClient:

    def __init__(self, deployment_api_uri, auth_token):
        self.__api_uri = deployment_api_uri
        self.__auth_token = auth_token
        self._api_session = ApiSession(auth_token, ssl_verify=True)

    def list_versions(self):
        list_versions_rsp = HttpUtils.send_request(url=urljoin(self.__api_uri, "versions"), method=HttpUtils.get,
                                                   api_session=self._api_session)
        return list_versions_rsp

    def get_version_def(self, target_version):
        version_def = HttpUtils.send_request(url=urljoin(self.__api_uri, "versions" + "/" + target_version),
                                             method=HttpUtils.get, api_session=self._api_session)
        return version_def

    def get_current_version(self, license_token):
        deployment_version = HttpUtils.send_request(url=urljoin(self.__api_uri, "deployments" + "/" + license_token),
                                                    method=HttpUtils.get, api_session=self._api_session)
        if deployment_version:
            return deployment_version['currentVersion']

    def get_deployment(self, license_token):
        deployment_resp = HttpUtils.send_request(url=urljoin(self.__api_uri, "deployments" + "/" + license_token),
                                                 method=HttpUtils.get, api_session=self._api_session)
        return deployment_resp

    def set_deployment_status(self, upgrade_ctx: UpgradeContext, status: DeploymentStatus, current_version=None):
        # XXX: grab current version always from context
        body = {
            "status": status.name.lower(),
            "currentVersion": current_version or upgrade_ctx.current_version()
        }
        HttpUtils.send_request(url=urljoin(self.__api_uri, "deployments" + "/" + upgrade_ctx.license_token()),
                               method=HttpUtils.put, data=body, api_session=self._api_session)

    def get_deployment_status(self, license_token: str):
        deployment_rs = self.get_deployment(license_token=license_token)
        if deployment_rs and 'status' in deployment_rs:
            return DeploymentStatus[deployment_rs['status'].upper()]


class MigrationError(RuntimeError):
    def __init__(self, current_version, target_version, exc_info):
        self.version = current_version
        self.target_version = target_version
        self.exc_info = exc_info


class MigrationToolClient:

    def __init__(self, migration_service_uri, auth_token=None):
        self.__api_uri = migration_service_uri
        self._api_session = ApiSession(auth_token=auth_token, ssl_verify=True)

    def execute_migration(self, upgrade_context, current_version, target_version):
        try:
            logger.info("Migrating to %s .... " % target_version)
            HttpUtils.send_request(url=urljoin(self.__api_uri, "migrate" + "/" + target_version),
                                   method=HttpUtils.post,
                                   params={"requestId": upgrade_context.run_id(),
                                           "currentVersion": upgrade_context.current_version()
                                           },
                                   api_session=self._api_session)
        except HTTPError as he:
            if he.response.status_code == 404:
                logger.warning("No schema migration to %s found. Only bits are going to be updated " % target_version)
            else:
                logger.exception("Error while migrating schema to %s " % target_version)
                raise RuntimeError(str(he))
        except ConnectionError as cer:
            logger.exception("Migration server is not running " + str(cer))
            raise RuntimeError("Migration server is not running " + str(cer))
        except RuntimeError as re:
            logger.exception("Error while migrating schema to %s " % target_version)
            raise re

        logger.info("Schema migration to %s version has been finished successfully" % target_version )


def is_supported(self, target_version):
        try:
            HttpUtils.send_request(url=urljoin(self.__api_uri, "migrate" + "/" + target_version),
                                   method=HttpUtils.get,
                                   api_session=self._api_session)
            return True
        except RuntimeError as re:
            logger.exception("Error while verifying migration support ", exc_info=re)
            raise re
