import os
from datetime import datetime

from compose.cli.docker_client import docker_client
from compose.config import config
from compose.config.environment import Environment
from compose.errors import NoHealthCheckConfigured, HealthCheckFailed
from compose.project import Project, NoSuchService
from log_config import logger

import iso8601


class ServiceId(object):
    def __init__(self, name: str):
        self._name = name

    def name(self):
        return self._name

    def __eq__(self, o: object) -> bool:
        return isinstance(o, ServiceId) and self._name.__eq__(o._name)

    def __hash__(self) -> int:
        return hash(self._name)

    def __str__(self) -> str:
        return "ServiceId(%s)".format(self._name)

    def to_dict(self):
        return {
            "name": self._name
        }

    def __str__(self) -> str:
        return "ServiceId(%s)" % self._name


class ServiceStatus(object):
    def __init__(self, service_id: ServiceId, status: str, is_healthy: bool, started_at: datetime = None,
                 timestamp: datetime = datetime.now(), display_name: str = None, details: dict = None):
        self.status = status
        self.timestamp = timestamp
        self.is_healthy = is_healthy
        self.started_at = started_at
        self.service_id = service_id
        self.display_name = display_name
        self.details = details or dict()

    def is_restarting(self):
        return self.status == 'Restarting'

    def is_running(self):
        return "up (healthy)" in self.status.lower()

    def to_dict(self):
        return {
            "status": self.status,
            "is_healthy": self.is_healthy,
            "started_at": self.started_at.isoformat(),
            "timestamp": self.timestamp.isoformat(),
            "service_id": self.service_id.name(),
            "display_name": self.display_name,
            "details": self.details
        }


class ServiceManager(object):

    def services(self):
        """
        return known services for this deployment
        :return: list of ServiceId
        """
        pass

    def start_service(self, service_id: ServiceId):
        """

        :param service_id: ServiceId
        :return: nothing or raises error
        """
        pass

    def stop_service(self, service_id: ServiceId):
        """

        :param service_id: ServiceId
        :return: nothing or raises error
        """
        pass

    def service_status(self, service_id: ServiceId):
        """
        Gets service status

        :param service_id:
        :return: ServiceStatus
        """
        pass

    def service_logs(self, service_id: ServiceId, tail: int=20, follow=False):
        pass


class DockerComposeServiceManager(ServiceManager):

    def __init__(self, base_dir: str, filenames: list):
        self._project_base_dir = base_dir
        self._compose_filenames = filenames
        logger.info("Creating docker-compose based service manager from base_dir %s ", base_dir)
        environment = Environment.from_env_file(base_dir)
        conf_details = config.find(base_dir=self._project_base_dir,
                                   filenames=self._compose_filenames,
                                   environment=environment)
        config_data = config.load(conf_details)
        api_client = docker_client(environment=environment)
        project_name = os.path.basename(base_dir)

        self._compose_project = Project.from_config(name=project_name, config_data=config_data, client=api_client)
        logger.debug("Service manager has been initialized")

    @staticmethod
    def _get_service_container(service):
        containers = service.containers(stopped=True)
        if len(containers) == 0:
            raise NoSuchService(name=service.name)
        service_container = containers[0]
        return service_container

    def get_service(self, service_id):
        return self._compose_project.get_service(service_id.name())

    def exec_cmd(self, service_id: ServiceId, cmd, demux=False):
        service = self._compose_project.get_service(service_id.name())
        container = DockerComposeServiceManager._get_service_container(service)

        command_id = container.create_exec(cmd)
        return container.start_exec(command_id["Id"], demux=demux)

    def restart(self, service_id: ServiceId):
        service = self._compose_project.get_service(service_id.name())
        container = DockerComposeServiceManager._get_service_container(service)
        return container.restart()

    @staticmethod
    def __get_container_status(service_container):
        return service_container.human_readable_state

    def services(self):
        return map(lambda service: ServiceId(name=service.name), self._compose_project.get_services())

    # throws NoService
    def start_service(self, service_id: ServiceId):
        service = self._compose_project.get_service(service_id.name())
        logger.info("Starting service %s", service_id.name())
        service.start()

    def stop_service(self, service_id: ServiceId):
        service = self._compose_project.get_service(service_id.name())
        logger.info("Stopping service %s", service.name)
        service_container = self._get_service_container(service)
        service_container.stop(timeout=service.stop_timeout(timeout=None))

    def service_status(self, service_id: ServiceId):
        service = self._compose_project.get_service(service_id.name())
        logger.debug("Getting status of service %s", service_id.name())

        service_container = self._get_service_container(service)
        started_at_str = service_container.get("State.StartedAt")
        display_name = service_container.labels.get("com.bpcs.conduit.displayName")
        started_at = None
        if started_at_str:
            started_at = iso8601.parse_date(started_at_str)
        healthy = None
        try:
            healthy = service.is_healthy()

        except NoHealthCheckConfigured as error:
            logger.warn("No healthcheck is configured for " + str(service_id) + " It is considered unhealthy")
            healthy = False
        except HealthCheckFailed:
            logger.debug("Healthcheck has failed for  " + str(service_id))
            healthy = False

        state = self.__get_container_status(service_container)
        return ServiceStatus(service_id=service_id, status=state,
                             is_healthy=healthy, started_at=started_at, display_name=display_name)

    def service_logs(self, service_id: ServiceId, tail: int=20, follow=False):

        service = self._compose_project.get_service(service_id.name())
        logger.debug("Getting logs of service %s", service_id.name())

        service_container = service.get_container()
        events = service_container.logs(tail=tail, follow=follow, stream=True)

        for event in events:
            yield event
