#!/usr/bin/env bash

function install_os_requirements {
    SCRIPT_PATH="$1"
    echo "Creating custom python environment at ${SCRIPT_PATH}/env "
    python3 -m venv ${SCRIPT_PATH}/env
    source ${SCRIPT_PATH}/env/bin/activate
    pip install --upgrade pip
    pip install envtpl ansible -q --no-input
}

function validate_os_requirements {
    if ! command -v envtpl &>/dev/null; then
      echo "Install envtpl using 'pip install envtpl' "
      exit 2
    fi

    if ! command -v ansible-playbook &>/dev/null; then
      echo "Install ansible using 'pip install ansible' "
      exit 2
    fi
}

function validate_gcloud_os_requirements {
    if ! command -v gcloud &>/dev/null; then
      echo "Install gcloud cli  https://cloud.google.com/sdk/docs/install "
      exit 1
    fi
}

function boolean() {
  case $1 in
  TRUE) echo true ;;
  true) echo true ;;
  FALSE) echo false ;;
  false) echo false ;;
  *)
    echo "Err: Unknown boolean value \"$1\"" 1>&2
    exit 1
    ;;
  esac
}

function yes_no_confirmation() {
  local MSG=$1
  local DEFAULT_ANSWER=$3
  if [[ -z "${NO_INPUT}" ]]; then
    while true; do
      read -p "$MSG" -r YN
      case $YN in
      [Yy]*)
        eval "$2=true"
        break
        ;;
      [Nn]*)
        eval "$2=false"
        break
        ;;
      *) echo "Please answer yes/y/Y or no/n/N." ;;
      esac
    done
  else
    if [[ -z "${DEFAULT_ANSWER}" ]]; then
      echo "ERROR: Enable to determine default value for prompt: $MSG"
      exit 6
    fi
    eval "$2=${DEFAULT_ANSWER}"
  fi
}

# Usage
#
#  prompt_non_empty_str_value "Enter deployment name: " DEPLOYMENT_NAME  "default-deployment-name"
#
function prompt_non_empty_str_value() {
  MSG=$1
  DEFAULT_ANSWER="$3"
  if [[ -z "${NO_INPUT}" ]]; then
    while true; do
      read -p "$MSG" -r PROMPT_VALUE
      if [[ -n "${PROMPT_VALUE}" ]]; then
        eval "$2=\"${PROMPT_VALUE}\""
        break
      else
        echo "Please enter non-empty string "
      fi
    done
  else
    if [[ -z "${DEFAULT_ANSWER}" ]]; then
      echo "ERROR: Enable to determine default value for prompt: $MSG"
      exit 6
    fi
    echo "Non-interactive mode is enabled, setting default value to ${DEFAULT_ANSWER} "
    eval "$2=\"${DEFAULT_ANSWER}\""
  fi
}

function prompt_str_value() {
  MSG=$1
  DEFAULT_ANSWER="$3"
  # if interactive mode and output variable is not defined
  if [[ -z "${NO_INPUT}" ]]; then
    read -p "$MSG" -r PROMPT_VALUE
    if [[ -n "${PROMPT_VALUE}" ]]; then
      eval "$2=\"${PROMPT_VALUE}\""
    else
      eval "$2=\"${DEFAULT_ANSWER}\""
    fi
  else
    if [[ -z "${DEFAULT_ANSWER}" ]]; then
      echo "ERROR: Enable to determine default value for prompt: $MSG"
      exit 6
    fi
    echo "Non-interactive mode is enabled, setting default value to ${DEFAULT_ANSWER} "
    eval "$2=\"${DEFAULT_ANSWER}\""
  fi
}

function parse_generic_install_parameters() {
  POSITIONAL=()
  while [[ $# -gt 0 ]]; do
    key="$1"

    case $key in
    -t | --type)
      export DEPLOYMENT_TYPE="$2"
      shift # past argument
      shift # past value
      ;;
    -n | --name)
      export DEPLOYMENT_NAME="$2"
      shift # past argument
      shift # past value
      ;;
    -r | --region | --location)
      export DEPLOYMENT_REGION="$2"
      shift # past argument
      shift # past value
      ;;
    -s | --size)
      export VM_COUNT="$2"
      shift # past argument
      shift # past value
      ;;
    *) # unknown option
      POSITIONAL+=("$1") # save it in an array for later
      shift # past argument
      ;;
    esac
  done
}

function parse_generic_conduit_parameters() {

  while [[ $# -gt 0 ]]; do
    key="$1"

    case $key in

    --domain-fqdn)
      export DEPLOYMENT_FQDN="$2"
      shift # past argument
      shift # past value
      ;;
    -v | --version)
      export BDE_VERSION="$2"
      shift # past argument
      shift # past value
      ;;
    --file-system)
      export CONDUIT_FS_TYPE="$2"
      shift # past argument
      shift # past value
      ;;
    --docker-user)
      export DOCKER_USER="$2"
      shift # past argument
      shift # past value
      ;;
    --docker-password)
      export DOCKER_PASSWORD="$2"
      shift # past argument
      shift # past value
      ;;
    --conduit-admin-name)
      export CONDUIT_ADMIN_NAME="$2"
      shift # past argument
      shift # past value
      ;;
    --conduit-admin-surname)
      export CONDUIT_ADMIN_SURNAME="$2"
      shift # past argument
      shift # past value
      ;;
    --conduit-admin-email)
      export CONDUIT_ADMIN_EMAIL="$2"
      shift # past argument
      shift # past value
      ;;
    --conduit-admin-password)
      export CONDUIT_ADMIN_PASSWORD="$2"
      shift # past argument
      shift # past value
      ;;
    --ssh-public-key)
      export SSH_PUBLIC_KEY_FILE="$2"
      shift # past argument
      shift # past value
      ;;
    --service-token)
      export CONDUIT_DEPLOYMENT_SERVICE_TOKEN="$2"
      shift # past argument
      shift # past value
      ;;

    esac
  done

}

function load_gcloud_uninstall_parameters() {
  while [[ $# -gt 0 ]]; do
    key="$1"

    case $key in
    -p | --project)
      export GCP_PROJECT_ID="$2"
      shift # past argument
      shift # past value
      ;;
    *) # unknown option
      shift # past argument
      ;;
    esac
  done
}

function prompt_generic_install_parameters() {
  if [ -z ${DEPLOYMENT_TYPE} ]; then


    echo "Available installation types are: "
    printf "\n\t[azure] - Creates resources and install Conduit on Azure"
    printf "\n\t[gcp] - Creates resources and install Conduit on Google Cloud Platform"
    printf "\n\t[on-premise] - Install Conduit on-premise\n\n"

    PS3='Please choose the deployment type: '
    options=("azure" "gcp" "on-premise")
    select opt in "${options[@]}"
    do
        case $opt in
            "azure")
                DEPLOYMENT_TYPE="azure"
                break
                ;;
            "gcp")
                DEPLOYMENT_TYPE="gcp"
                break
                ;;
            "on-premise")
                DEPLOYMENT_TYPE="on-premise"
                break
                ;;
            *)
                echo "invalid option $REPLY "
                ;;
        esac
    done

    export DEPLOYMENT_TYPE
  fi

  if [ -z ${VM_COUNT} ]; then
    prompt_str_value "Enter the number of instances to deploy [minimum 3]: " VM_COUNT 3
    export VM_COUNT
  fi

}

function print_array_on_each_line() {
  local multiline=("$1")
  local arr=()
  while read -r line; do
    arr+=("$line")
  done <<<"$multiline"
  local i

  printf "\n"
  for ((i = 0; i < ${#arr[@]}; i++)); do
    printf "\t${arr[$i]}\n"
  done
  printf "\n"
}

function load_gcloud_uninstall_parameters_input() {
  if [ -z ${GCP_PROJECT_ID} ]; then
    AVAILABLE_PROJECTS=$(gcloud projects list --format="get(PROJECT_ID)")
    echo "Choose one of the following available projects:"
    print_array_on_each_line "${AVAILABLE_PROJECTS}"
    DEFAULT_PROJECT=$(gcloud config get-value project)
    prompt_non_empty_str_value "Enter the Google project id. Default [${DEFAULT_PROJECT}]: " TMP_GCP_PROJECT_ID $DEFAULT_PROJECT
    export GCP_PROJECT_ID=$TMP_GCP_PROJECT_ID
  fi

  if [ -z ${DEPLOYMENT_NAME} ]; then
    prompt_non_empty_str_value "Enter the deployment name: " DEPLOYMENT_NAME
    export DEPLOYMENT_NAME
  fi

  if [ -z ${DEPLOYMENT_REGION} ]; then
    AVAILABLE_REGIONS=$(gcloud compute regions list --format="get(NAME)")
    echo "Choose one of the following available regions:"
    print_array_on_each_line "${AVAILABLE_REGIONS}"
    prompt_non_empty_str_value "Enter the deployment region: " DEPLOYMENT_REGION
    export DEPLOYMENT_REGION
  fi
}

function load_gcloud_parameters() {

    POSITIONAL=()
    while [[ $# -gt 0 ]]; do
        key="$1"

        case $key in
         -p | --project)
          export GCP_PROJECT_ID="$2"
          shift # past argument
          shift # past value
          ;;
         -c|--certificate)
          export SSL_CERTIFICATE_NAME="$2"
          shift # past argument
          shift # past value
          ;;
          --certificate-public)
          export SSL_CERTIFICATE_PUBLIC_FILE="$2"
          shift # past argument
          shift # past value
          ;;
          --certificate-private)
          export SSL_CERTIFICATE_PRIVATE_FILE="$2"
          shift # past argument
          shift # past value
          ;;
          --ssh-private-key)
          export SSH_PRIVATE_KEY_FILE="$2"
          shift # past argument
          shift # past value
          ;;
          --session-affinity)
          export SESSION_AFFINITY="$2"
          shift # past argument
          shift # past value
          ;;
          --service-token)
          export CONDUIT_DEPLOYMENT_SERVICE_TOKEN="$2"
          shift # past argument
          shift # past value
          ;;
        *) # unknown option
          POSITIONAL+=("$1") # save it in an array for later
          shift # past argument
          ;;
      esac
    done
}

function load_gcloud_parameters_input {
    if [ -z ${GCP_PROJECT_ID} ]; then
        AVAILABLE_PROJECTS=`gcloud projects list --format="get(PROJECT_ID)"`
        echo "Choose one of the following available projects:"
        print_array_on_each_line "${AVAILABLE_PROJECTS}"
        DEFAULT_PROJECT=`gcloud config get-value project`
        read -e -p "Enter the Google project id. Default [${DEFAULT_PROJECT}]: " TMP_GCP_PROJECT_ID
        if [ ! -z ${TMP_GCP_PROJECT_ID} ]; then
            export GCP_PROJECT_ID=$TMP_GCP_PROJECT_ID
        else
            export GCP_PROJECT_ID=$DEFAULT_PROJECT
        fi
    fi

    if [ -z ${DEPLOYMENT_NAME} ]; then
        read -e -p "Enter the deployment name: " DEPLOYMENT_NAME
        export DEPLOYMENT_NAME
    fi

    if [ -z ${DEPLOYMENT_REGION} ]; then
        AVAILABLE_REGIONS=`gcloud compute regions list --format="get(NAME)"`
        echo "Choose one of the following available regions:"
        print_array_on_each_line "${AVAILABLE_REGIONS}"
        read -e -p "Enter the deployment region: " DEPLOYMENT_REGION
        export DEPLOYMENT_REGION
    fi

    if [ -z ${SSL_CERTIFICATE_NAME} ]; then
        while true; do
        read -p "Do you have Google cloud registered SSL certificate? [Yes|No]:" yn
        case $yn in
            [Yy]* )
                read -e -p "Enter the SSL certificate name: " SSL_CERTIFICATE_NAME;
                export SSL_CERTIFICATE_NAME
            break;;
            [Nn]* )
                read -e -p "Enter the public SSL certificate file path: " SSL_CERTIFICATE_PUBLIC_FILE;
                read -e -p "Enter the private SSL certificate file path: " SSL_CERTIFICATE_PRIVATE_FILE;
                export SSL_CERTIFICATE_PUBLIC_FILE
                export SSL_CERTIFICATE_PRIVATE_FILE
            break;;
            * ) echo "Please answer yes or no.";;
        esac
    done
    fi


    if [ -z ${DEPLOYMENT_FQDN} ]; then
        read -e -p "Enter the Fully Qualified Domain Name (FQDN) of Conduit deployment: " DEPLOYMENT_FQDN
        export DEPLOYMENT_FQDN
    fi

    if [ -z ${BDE_VERSION} ]; then
        read -e -p "Enter the Conduit build version [Default 2.5.0]: " TMP_BDE_VERSION
        if [ ! -z ${TMP_BDE_VERSION} ]; then
            export BDE_VERSION=TMP_BDE_VERSION
        else
            export BDE_VERSION="2.5.0"
        fi
    fi

    if [ -z ${CONDUIT_FS_TYPE} ]; then
        echo "Conduit file system available options: [file|hdfs|abfs|s3|gcs]"
        read -e -p "Enter the Conduit file system type [Default gcs]: " TMP_CONDUIT_FS_TYPE
        if [ ! -z ${TMP_CONDUIT_FS_TYPE} ]; then
            export CONDUIT_FS_TYPE=$TMP_CONDUIT_FS_TYPE
        else
            export CONDUIT_FS_TYPE="gcs"
        fi
    fi

    if [ -z ${CONDUIT_DEPLOYMENT_SERVICE_TOKEN} ]; then
        read -s -p "Enter your conduit deployment service authentication token: " CONDUIT_DEPLOYMENT_SERVICE_TOKEN
        export CONDUIT_DEPLOYMENT_SERVICE_TOKEN
    fi

    if [ -z ${DOCKER_USER} ]; then
        read -e -p "Enter the docker username: " DOCKER_USER
        export DOCKER_USER
    fi

    # TODO - password input
    if [ -z ${DOCKER_PASSWORD} ]; then
        read -s -p "Enter the docker password: " DOCKER_PASSWORD
        export DOCKER_PASSWORD
    fi

    if [ -z ${CONDUIT_ADMIN_NAME} ]; then
        read -e -p "Enter the Conduit admin name: " CONDUIT_ADMIN_NAME
        export CONDUIT_ADMIN_NAME
    fi

    if [ -z ${CONDUIT_ADMIN_SURNAME} ]; then
        read -e -p "Enter the Conduit admin last name: " CONDUIT_ADMIN_SURNAME
        export CONDUIT_ADMIN_SURNAME
    fi

    if [ -z ${CONDUIT_ADMIN_EMAIL} ]; then
        read -e -p "Enter the Conduit admin email address: " CONDUIT_ADMIN_EMAIL
        export CONDUIT_ADMIN_EMAIL
    fi

    if [ -z ${CONDUIT_ADMIN_PASSWORD} ]; then
        read -s -p "Enter the Conduit admin email password: " CONDUIT_ADMIN_PASSWORD
        export CONDUIT_ADMIN_PASSWORD
    fi

    if [ -z ${SSH_PUBLIC_KEY_FILE} ]; then
        read -e -p "Enter the ssh public key file path: " SSH_PUBLIC_KEY_FILE
        export SSH_PUBLIC_KEY_FILE
    fi

    if [ -z ${SSH_PRIVATE_KEY_FILE} ]; then
        read -e -p "Enter the ssh private key file path: " SSH_PRIVATE_KEY_FILE
        export SSH_PRIVATE_KEY_FILE
    fi

    if [ -z ${SESSION_AFFINITY} ]; then
        echo "Load balancer session affinity available options are: [CLIENT_IP, CLIENT_IP_PORT_PROTO, CLIENT_IP_PROTO, GENERATED_COOKIE, HEADER_FIELD, HTTP_COOKIE, NONE]"
        echo "More info about session affinity on https://cloud.google.com/load-balancing/docs/backend-service#session_affinity"
        read -e -p "Choose the load balancer session affinity [Default CLIENT_IP] :" TMP_SESSION_AFFINITY
        if [ ! -z ${TMP_SESSION_AFFINITY} ]; then
            export SESSION_AFFINITY=$TMP_SESSION_AFFINITY
        else
            export SESSION_AFFINITY="CLIENT_IP"
        fi
    fi
}

function validate_gcloud_uninstall_parameters_are_defined {
    if [ -z ${DEPLOYMENT_NAME} ]; then
       echo "Deployment name cannot be empty. Exiting now"
       exit 1
    fi
    if [ ${#DEPLOYMENT_NAME} -ge 16 ]; then
        echo "Deployment name is longer than 16 characters. Exiting now"
        exit 1
    fi

    NAMED_REGION=`gcloud compute regions list --filter="name=('${DEPLOYMENT_REGION}')" --format="get(NAME)"`
    if [ -z ${NAMED_REGION} ]; then
       echo "Invalid deployment region: ${DEPLOYMENT_REGION}. Please check available regions at https://cloud.google.com/compute/docs/regions-zones"
       exit 1
    fi

    NAMED_PROJECT=`gcloud projects list --filter="PROJECT_ID=('${GCP_PROJECT_ID}')" --format="get(PROJECT_ID)"`
    if [ -z ${NAMED_PROJECT} ]; then
       echo "No such GCP project id: ${GCP_PROJECT_ID}. Exiting now"
       exit 1
    fi
}

function validate_gcloud_parameters_are_defined {
    echo "Validating deployment name"
    if [ -z ${DEPLOYMENT_NAME} ]; then
       echo "Deployment name cannot be empty. Exiting now"
       exit 1
    fi
    if [ ${#DEPLOYMENT_NAME} -ge 16 ]; then
        echo "Deployment name is longer than 16 characters. Exiting now"
        exit 1
    fi
    NAMED_DEPLOYMENT_NAME=`gcloud compute networks list --filter="name=('${VNET}')" --format="get(NAME)"`
    if [ ! -z ${NAMED_DEPLOYMENT_NAME} ]; then
       echo "The deployment: ${DEPLOYMENT_NAME} already exists. Please Choose a different name. Exiting now"
       exit 1
    fi

    echo "Validating deployment region"
    NAMED_REGION=`gcloud compute regions list --filter="name=('${DEPLOYMENT_REGION}')" --format="get(NAME)"`
    if [ -z ${NAMED_REGION} ]; then
       echo "Invalid deployment region: ${DEPLOYMENT_REGION}. Please check available regions at https://cloud.google.com/compute/docs/regions-zones"
       exit 1
    fi

    echo "Validating google project"
    NAMED_PROJECT=`gcloud projects list --filter="PROJECT_ID=('${GCP_PROJECT_ID}')" --format="get(PROJECT_ID)"`
    if [ -z ${NAMED_PROJECT} ]; then
       echo "No such GCP project id: ${GCP_PROJECT_ID}. Exiting now"
       exit 1
    fi

    echo "Validating SSL certificate"
    if [ -z ${SSL_CERTIFICATE_NAME} ]; then
        if [ ! -f "${SSL_CERTIFICATE_PUBLIC_FILE}" ]; then
            echo "Local file ${SSL_CERTIFICATE_PUBLIC_FILE} does not exists. Exiting now"
            exit 1
        fi
        if [ ! -f "${SSL_CERTIFICATE_PRIVATE_FILE}" ]; then
            echo "Local file ${SSL_CERTIFICATE_PRIVATE_FILE} does not exists. Exiting now"
            exit 1
        fi
    else
        NAMED_CERTIFICATE=`gcloud compute ssl-certificates list --filter="name=('${SSL_CERTIFICATE_NAME}')" --format="get(NAME)"`
        if [ -z ${NAMED_CERTIFICATE} ]; then
           echo "No such SSL certificate registered on GCP: ${SSL_CERTIFICATE_NAME}. Exiting now"
           exit 1
        fi
    fi

    echo "Validating number of virtual machines"
    if [[ $VM_COUNT =~ '!^[0-9]+$' ]] ; then
       echo "Invalid input for the number of virtual machines: ${VM_COUNT}. Exiting now"
       exit 1
    fi

    echo "Validating conduit file system type"
    if [[ ! $CONDUIT_FS_TYPE =~ ^(file|hdfs|abfs|s3|gcs)$ ]]; then
        echo "Invalid Chosen File System type: ${CONDUIT_FS_TYPE}, expected one of [file|hdfs|abfs|s3|gcs]. Exiting now"
        exit 1
    fi

    echo "Validating RSA access keys"
    if [ ! -f "${SSH_PUBLIC_KEY_FILE}" ]; then
        echo "SSH_PUBLIC_KEY_FILE does not exists. Exiting now"
        exit 1
    fi
    if [ ! -f "${SSH_PRIVATE_KEY_FILE}" ]; then
        echo "SSH_PRIVATE_KEY_FILE does not exists. Exiting now"
        exit 1
    fi

    echo "Validating load balancer session affinity"
    if [[ ! $SESSION_AFFINITY =~ ^(CLIENT_IP|CLIENT_IP_PORT_PROTO|CLIENT_IP_PROTO|GENERATED_COOKIE|HEADER_FIELD|HTTP_COOKIE|NONE)$ ]]; then
        echo "Invalid Chosen Session affinity: ${SESSION_AFFINITY}, expected one of [CLIENT_IP, CLIENT_IP_PORT_PROTO, CLIENT_IP_PROTO, GENERATED_COOKIE, HEADER_FIELD, HTTP_COOKIE, NONE]. Exiting now"
        exit 1
    fi

    echo "Validating conduit deployment service authentication token"
    if [ -z "${CONDUIT_DEPLOYMENT_SERVICE_TOKEN}" ]; then
        echo "Conduit deployment service authentication token cannot be empty. Exiting now"
        exit 1
    fi
}

function set_gcloud_resources_names {
    export TAG_ALLOW_SSH="${DEPLOYMENT_NAME}-allow-ssh"
    export VNET="${DEPLOYMENT_NAME}-vnet"
    export VNET_SUBNET="${VNET}-sub"

    export ROUTER="${DEPLOYMENT_NAME}-rt"
    export NAT="${ROUTER}-nat"
    export NAT_IP="${NAT}-ip"

    export INSTANCE_TEMPLATE="${DEPLOYMENT_NAME}-i-template"
    export INSTANCE_VM="${DEPLOYMENT_NAME}-vm"
    export JUMPBOX_IP="${DEPLOYMENT_NAME}-jb-ip"

    export VNET_FIREWALL_ALLOW_INTERNAL="${VNET}-fw-allow-internal"
    export VNET_FIREWALL_ALLOW_EXTERNAL="${VNET}-fw-allow-external"
    export VNET_FIREWALL_ALLOW_SSH="${VNET}-fw-allow-ssh"
    export VNET_FIREWALL_HEALTH_CHECK="${VNET}-fw-health-check"

    export LB="${DEPLOYMENT_NAME}-lb"
    export LB_IP="${LB}-ip"
    export LB_HEALTH_CHECK="${LB}-hc"
    export LB_HEALTH_CHECK_SSL="${LB}-hc-ssl"
    export LB_BACKEND_SERVICE="${LB}-bs"
    export LB_BACKEND_SERVICE_SSL="${LB}-bs-ssl"
    export LB_URL_MAPS_HTTPS="${LB}-map-https"
    export LB_URL_MAPS_HTTP="${LB}-map-http"
    export LB_TARGET_PROXY_SSL="${LB}-proxy-ssl"
    export LB_TARGET_PROXY_HTTP="${LB}-proxy-http"
    export LB_TARGET_PROXY_HTTPS="${LB}-proxy-https"
    export LB_FORWARD_RULE_HTTP="${LB}-fwd-rule-http"
    export LB_FORWARD_RULE_HTTPS="${LB}-fwd-rule-https"
    export LB_FORWARD_RULE_SSL="${LB}-fwd-rule-ssl"
}
